@@ -1,11 +1,13 @@
-use crate::{BlockExtra, FsBlock, Periodic};
+use crate::{BlockExtra, FsBlock, PeriodCounter, Periodic};
 use bitcoin::blockdata::constants::genesis_block;
 use bitcoin::{BlockHash, Network};
 use log::{info, warn};
 use std::collections::HashMap;
 use std::convert::TryInto;
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::mpsc::Receiver;
 use std::sync::mpsc::SyncSender;
+use std::sync::Arc;
 use std::thread::JoinHandle;
 use std::time::{Duration, Instant};
 
@@ -92,6 +94,8 @@ impl Reorder {
     pub fn new(
         network: Network,
         max_reorg: u8,
+        stop_at_height: Option<u32>,
+        early_stop: Arc<AtomicBool>,
         receiver: Receiver<Option<Vec<FsBlock>>>,
         sender: SyncSender<Option<BlockExtra>>,
     ) -> Self {
@@ -101,6 +105,8 @@ impl Reorder {
         let mut periodic = Periodic::new(Duration::from_secs(60));
         Self {
             join: Some(std::thread::spawn(move || {
+                let mut bench = PeriodCounter::new(Duration::from_secs(10));
+
                 let mut busy_time = 0u128;
                 let mut now = Instant::now();
                 let mut last_height = 0;
@@ -110,7 +116,10 @@ impl Reorder {
                     now = Instant::now();
                     match received {
                         Some(raw_blocks) => {
-                            for raw_block in raw_blocks {
+                            if early_stop.load(Ordering::SeqCst) {
+                                break;
+                            }
+                            'outer: for raw_block in raw_blocks {
                                 if periodic.elapsed() {
                                     info!(
                                         "reorder receive:{} size:{} follows:{} next:{}",
@@ -135,6 +144,7 @@ impl Reorder {
                                 while let Some(block_to_send) = blocks.remove(&next) {
                                     let mut block_extra: BlockExtra =
                                         block_to_send.try_into().unwrap();
+
                                     busy_time += now.elapsed().as_nanos();
                                     next = block_extra.next[0];
                                     block_extra.height = height;
@@ -142,10 +152,29 @@ impl Reorder {
                                     blocks
                                         .blocks
                                         .remove(&block_extra.block.header.prev_blockhash);
+
+                                    bench.count_block(&block_extra.block);
+                                    if let Some(stats) = bench.period_elapsed() {
+                                        info!(
+                                            "# {:7} {} {:?}",
+                                            block_extra.height,
+                                            block_extra.block_hash,
+                                            block_extra.fee()
+                                        );
+                                        info!("{}", stats);
+                                    }
+
                                     sender.send(Some(block_extra)).unwrap();
                                     height += 1;
                                     now = Instant::now();
                                     last_height = height;
+                                    if let Some(stop_at_height) = stop_at_height {
+                                        if height > stop_at_height {
+                                            info!("reached height: {}", stop_at_height);
+                                            early_stop.store(true, Ordering::Relaxed);
+                                            break 'outer;
+                                        }
+                                    }
                                 }
                             }
                         }
